@@ -18,7 +18,6 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-
 #define __SND_OSS_COMPAT__
 #define SND_MAIN_OBJECT_FILE
 #include "../include/driver.h"
@@ -34,6 +33,7 @@ Card: ADI SoundComm AD1815\n\
 Card: Highscreen Sound-Boostar 16 3D\n\
 ISAPNP: MDK1605=ADS7180,ADS7181\n\
 ISAPNP: ADS7150=ADS7150,ADS7151\n\
+ISAPNP: ADS7181=ADS7180,ADS7181\n\
 ");
 
 int snd_index[SND_CARDS] = SND_DEFAULT_IDX;		/* Index 1-MAX */
@@ -99,6 +99,10 @@ static unsigned int snd_card_ad1816a_pnpids[] = {
 	(ISAPNP_VENDOR('A','D','S')<<16)|ISAPNP_DEVICE(0x7150),    /* device */
 	(ISAPNP_VENDOR('A','D','S')<<16)|ISAPNP_FUNCTION(0x7150),  /* audio */
 	(ISAPNP_VENDOR('A','D','S')<<16)|ISAPNP_FUNCTION(0x7151),  /* mpu401 */
+	/* Analog Devices AD1816A - added by Kenneth Platz <kxp@atl.hp.com> */
+	(ISAPNP_VENDOR('A','D','S')<<16)|ISAPNP_DEVICE(0x7181),    /* device */
+	(ISAPNP_VENDOR('A','D','S')<<16)|ISAPNP_FUNCTION(0x7180),  /* audio */
+	(ISAPNP_VENDOR('A','D','S')<<16)|ISAPNP_FUNCTION(0x7181),  /* mpu401 */
 	0	/* end */
 };
 #endif	/* __ISAPNP__ */
