@@ -1104,7 +1104,7 @@ struct snd_oss_mixer_info_obsolete {
 #define SND_PCM_FMT_IEC958_SUBFRAME	SND_PCM_FMT_IEC958_SUBFRAME_BE
 #endif
 
-#define SND_PCM_RATE_PLL		(1<<0)		/* programmable PLL */
+#define SND_PCM_RATE_CONTINUOUS		(1<<0)		/* continuous range */
 #define SND_PCM_RATE_KNOT		(1<<1)		/* supports more non-continuos rates */
 #define SND_PCM_RATE_8000		(1<<2)		/* 8000Hz */
 #define SND_PCM_RATE_11025		(1<<3)		/* 11025Hz */
@@ -1292,7 +1292,7 @@ typedef struct snd_pcm_channel_info {
 	unsigned int rates;		/* hardware rates */
 	int min_rate;			/* min rate (in Hz) */
 	int max_rate;			/* max rate (in Hz) */
-	int min_voices;			/* min voices (probably always 1) */
+	int min_voices;			/* min voices */
 	int max_voices;			/* max voices */
 	int buffer_size;		/* max buffer size in bytes */
 	int min_fragment_size;		/* min fragment size in bytes */
@@ -1430,6 +1430,7 @@ typedef struct {
 #define SND_PCM_LB_TYPE_DATA		0	/* sample data */
 #define SND_PCM_LB_TYPE_FORMAT		1	/* format change */
 #define SND_PCM_LB_TYPE_POSITION	2	/* position change (in bytes) */
+#define SND_PCM_LB_TYPE_SILENCE		3	/* silence (no data) */
 
 typedef struct snd_pcm_loopback_header {
 	int size;			/* block size in bytes */
