@@ -903,7 +903,7 @@ snd_info_entry_t *snd_info_create_device(const char *name, unsigned int number,
 	up(&info_mutex);
 #ifdef CONFIG_DEVFS_FS
 	sprintf(dname, "snd/%s", name);
-	devfs_register(NULL, dname, 0, DEVFS_FL_TOLERANT,
+	devfs_register(NULL, dname, 0, DEVFS_FL_DEFAULT,
 			major, minor, mode, 0, 0,
 			&snd_fops, NULL);
 #endif
