@@ -303,7 +303,7 @@ static int snd_pcm_lib_ioctl_params(void *private_data,
 #ifdef DEBUG_PARAMS
 	printk("params - rate\n");
 #endif
-	if (hw->rates & (SND_PCM_RATE_PLL|SND_PCM_RATE_KNOT)) {
+	if (hw->rates & (SND_PCM_RATE_CONTINUOUS|SND_PCM_RATE_KNOT)) {
 		if (runtime->format.rate < hw->min_rate ||
 		    runtime->format.rate > hw->max_rate)
 			return -EINVAL;
@@ -413,6 +413,8 @@ static int snd_pcm_lib_ioctl_status(void *private_data,
 		if (status->count < 0) {
 			status->count = 0;
 			stop++;
+		} else if (status->count > runtime->buf.stream.queue_size) {
+			status->count = runtime->buf.stream.queue_size;
 		}
 		if ((tmpu = runtime->hw->transfer_block_size) > 1) {
 			if (ptr > tmpu) {
@@ -684,7 +686,11 @@ int snd_pcm_playback_empty(snd_pcm_subchn_t *subchn)
 	spin_lock_irqsave(&runtime->lock, flags);
 	if (runtime->mode == SND_PCM_MODE_BLOCK) {
 		if (runtime->flags & SND_PCM_FLG_MMAP) {
-			result = runtime->mmap_control->fragments[*runtime->frag_tail].data ? 0 : 1;
+			if (*runtime->status == SND_PCM_STATUS_UNDERRUN) {
+				result = 1;
+			} else {
+				result = runtime->mmap_control->fragments[*runtime->frag_tail].data ? 0 : 1;
+			}
 		} else {
 			result = runtime->frag_used ? 0 : 1;
 		}
@@ -1026,15 +1032,19 @@ void snd_pcm_transfer_done(snd_pcm_subchn_t *subchn)
 		runtime->transfer_ack_end(subchn);
 }
 
-static int snd_pcm_lib_fill(snd_pcm_subchn_t *subchn, unsigned int ptr)
+static int snd_pcm_lib_fill(snd_pcm_subchn_t *subchn, unsigned int ptr,
+			    void (*silence)(snd_pcm_subchn_t *subchn,
+			    		    unsigned int dstoff,
+			    		    unsigned int size,
+			    		    unsigned char silencebyte))
 {
 	snd_pcm_runtime_t *runtime = subchn->runtime;
 	unsigned int xfree, len;
-	unsigned char silence;
+	unsigned char silencebyte;
 	unsigned long flags;
 	
 	spin_lock_irqsave(&runtime->lock, flags);
-	silence = snd_pcm_lib_silence(subchn);
+	silencebyte = snd_pcm_lib_silence(subchn);
 	if (runtime->fill_position < runtime->position)
 		runtime->fill_position = runtime->position;
 	xfree = (ptr + runtime->buf.stream.queue_size) - runtime->position;
@@ -1047,18 +1057,14 @@ static int snd_pcm_lib_fill(snd_pcm_subchn_t *subchn, unsigned int ptr)
 	if (xfree >= runtime->buf.stream.queue_size) {
 		spin_unlock_irqrestore(&runtime->lock, flags);
 		snd_pcm_transfer_stop(subchn, SND_PCM_STATUS_UNDERRUN);
-		return -EIO;
+		return -EPIPE;
 	}
 	while (xfree > 0 && *runtime->status != SND_PCM_STATUS_UNDERRUN) {
 		len = xfree;
 		if ((runtime->fill_position % runtime->buf.stream.queue_size) + len > runtime->buf.stream.queue_size)
 			len = runtime->buf.stream.queue_size - (runtime->fill_position % runtime->buf.stream.queue_size);
 		spin_unlock_irqrestore(&runtime->lock, flags);
-		if (runtime->hw_memset) {
-			runtime->hw_memset(subchn, -1, runtime->fill_position % runtime->buf.stream.queue_size, silence, len);
-		} else {
-			memset(runtime->dma_area->buf + (runtime->fill_position % runtime->buf.stream.queue_size), silence, len);
-		}
+		silence(subchn, runtime->fill_position % runtime->buf.stream.queue_size, len, silencebyte);
 		spin_lock_irqsave(&runtime->lock, flags);
 		xfree -= len;
 		runtime->fill_position += len;
@@ -1068,8 +1074,24 @@ static int snd_pcm_lib_fill(snd_pcm_subchn_t *subchn, unsigned int ptr)
 	return 0;
 }
 
-static int snd_pcm_lib_write_transfer(snd_pcm_subchn_t *subchn, unsigned int dstoff,
-				      const void *src, unsigned int srcoff, unsigned int size)
+static void snd_pcm_lib_write_silence(snd_pcm_subchn_t *subchn,
+				      unsigned int dstoff,
+				      unsigned int size,
+				      unsigned char silencebyte)
+{
+	snd_pcm_runtime_t *runtime = subchn->runtime;
+
+	if (runtime->hw_memset) {
+		runtime->hw_memset(subchn, -1, dstoff, silencebyte, size);
+	} else {
+		memset(runtime->dma_area->buf + dstoff, silencebyte, size);
+	}
+}
+
+static int snd_pcm_lib_write_transfer(snd_pcm_subchn_t *subchn,
+				      unsigned int dstoff,
+				      const void *src, unsigned int srcoff,
+				      unsigned int size)
 {
 	snd_pcm_runtime_t *runtime = subchn->runtime;
 	int err;
@@ -1105,7 +1127,7 @@ static long snd_pcm_lib_write_block(snd_pcm_subchn_t *subchn, const void *src, u
 			if (runtime->start_mode == SND_PCM_START_FULL)
 				snd_pcm_kernel_playback_ioctl(subchn, SND_PCM_IOCTL_CHANNEL_GO, 0);
 			if (*runtime->status != SND_PCM_STATUS_RUNNING)
-				return result > 0 ? result : -EIO;
+				return result > 0 ? result : -EPIPE;
 			if (nonblock)
 				return result > 0 ? result : -EAGAIN;
 			timeout = interruptible_sleep_on_timeout(&runtime->sleep, 10 * HZ);
@@ -1143,7 +1165,9 @@ static long snd_pcm_lib_write_block(snd_pcm_subchn_t *subchn, const void *src, u
 
 static long snd_pcm_lib_write_stream(snd_pcm_subchn_t *subchn, const void *src, unsigned long count, int nonblock,
 				     int (*transfer)(snd_pcm_subchn_t *subchn, unsigned int dstoff,
-						     const void *src, unsigned int srcoff, unsigned int size))
+						     const void *src, unsigned int srcoff, unsigned int size),
+				     void (*silence)(snd_pcm_subchn_t *subchn, unsigned int dstoff,
+				     		     unsigned int size, unsigned char silencebyte))
 {
 	snd_pcm_runtime_t *runtime = subchn->runtime;
 	unsigned long flags;
@@ -1174,13 +1198,13 @@ static long snd_pcm_lib_write_stream(snd_pcm_subchn_t *subchn, const void *src,
 			if (runtime->stop_mode != SND_PCM_STOP_ROLLOVER) {
 				spin_unlock_irqrestore(&runtime->lock, flags);
 				snd_pcm_transfer_stop(subchn, SND_PCM_STATUS_UNDERRUN);
-				return result > 0 ? result : -EIO;
+				return result > 0 ? result : -EPIPE;
 			}
 		}
 	} else {
 		if (*runtime->status == SND_PCM_STATUS_UNDERRUN) {
 			spin_unlock_irqrestore(&runtime->lock, flags);
-			return -EIO;
+			return -EPIPE;
 		}
 		ptr = 0;
 	}
@@ -1207,7 +1231,7 @@ static long snd_pcm_lib_write_stream(snd_pcm_subchn_t *subchn, const void *src,
 			return result > 0 ? result : err;
 		spin_lock_irqsave(&runtime->lock, flags);
 		if (*runtime->status == SND_PCM_STATUS_UNDERRUN) {
-			result = result > 0 ? result : -EIO;
+			result = result > 0 ? result : -EPIPE;
 			break;
 		}
 		offset += size;
@@ -1219,7 +1243,7 @@ static long snd_pcm_lib_write_stream(snd_pcm_subchn_t *subchn, const void *src,
 	}
 	spin_unlock_irqrestore(&runtime->lock, flags);
 	if (result >= 0 && runtime->buf.stream.fill != SND_PCM_FILL_NONE) {
-		if ((err = snd_pcm_lib_fill(subchn, ptr)) < 0)
+		if ((err = snd_pcm_lib_fill(subchn, ptr, silence)) < 0)
 			return result > 0 ? result : err;
 	}
 	if (result > 0 && runtime->start_mode == SND_PCM_START_DATA)
@@ -1241,7 +1265,7 @@ long snd_pcm_lib_write(snd_pcm_subchn_t *subchn, const char *buf, long count)
 	snd_debug_check(runtime == NULL, -ENXIO);
 	snd_debug_check(runtime->dma_area == NULL, -EINVAL);
 	if (*runtime->status < SND_PCM_STATUS_PREPARED)
-		return -EINVAL;
+		return -EBADFD;
 
 	snd_debug_check(subchn->ffile == NULL, -ENXIO);
 	nonblock = !!(subchn->ffile->f_flags & O_NONBLOCK);
@@ -1258,15 +1282,42 @@ long snd_pcm_lib_write(snd_pcm_subchn_t *subchn, const char *buf, long count)
 #endif
 
 	if (runtime->mode == SND_PCM_MODE_BLOCK) {
-		return snd_pcm_lib_write_block(subchn, buf, count, nonblock, snd_pcm_lib_write_transfer);
+		return snd_pcm_lib_write_block(subchn, buf, count, nonblock,
+					       snd_pcm_lib_write_transfer);
 	} else if (runtime->mode == SND_PCM_MODE_STREAM) {
-		return snd_pcm_lib_write_stream(subchn, buf, count, nonblock, snd_pcm_lib_write_transfer);
+		return snd_pcm_lib_write_stream(subchn, buf, count, nonblock,
+						snd_pcm_lib_write_transfer,
+						snd_pcm_lib_write_silence);
 	}
 	return -EIO;
 }
 
-static int snd_pcm_lib_writev_transfer(snd_pcm_subchn_t *subchn, unsigned int dstoff,
-				       const void *src, unsigned int srcoff, unsigned int size)
+static void snd_pcm_lib_writev_silence(snd_pcm_subchn_t *subchn,
+				       unsigned int dstoff,
+				       unsigned int size,
+				       unsigned char silencebyte)
+{
+	snd_pcm_runtime_t *runtime = subchn->runtime;
+	int voices = runtime->format.voices;
+	int v;
+
+	size /= voices;
+	dstoff /= voices;
+	if (runtime->hw_memset) {
+		for (v = 0; v < voices; v++)
+			runtime->hw_memset(subchn, v, dstoff, silencebyte, size);
+	} else {
+		/* default silence behaviour */
+		int buf_size = runtime->buf.stream.queue_size / voices;
+		for (v = 0; v < voices; v++)
+			memset(runtime->dma_area->buf + (v * buf_size) + dstoff, silencebyte, size);
+	}
+}
+
+static int snd_pcm_lib_writev_transfer(snd_pcm_subchn_t *subchn,
+				       unsigned int dstoff,
+				       const void *src, unsigned int srcoff,
+				       unsigned int size)
 {
 	snd_pcm_runtime_t *runtime = subchn->runtime;
 	int err;
@@ -1274,16 +1325,35 @@ static int snd_pcm_lib_writev_transfer(snd_pcm_subchn_t *subchn, unsigned int ds
 	int voices = runtime->format.voices;
 	int v;
 	size /= voices;
-	for (v = 0; v < voices; ++v) {
-		if (runtime->hw_memcpy) {
-			if ((err = runtime->hw_memcpy(subchn, v, dstoff, bufs[v] + srcoff, size)) < 0)
-				return err;
-		} else {
-			if (copy_from_user(runtime->dma_area->buf + dstoff, bufs[v] + srcoff, size))
-				return -EFAULT;
-			dstoff += size;
+	srcoff /= voices;
+	dstoff /= voices;
+	if (runtime->hw_memcpy) {
+		snd_debug_check(runtime->hw_memset == NULL, -EINVAL);
+		for (v = 0; v < voices; ++v) {
+			if (bufs[v] == NULL) {
+				if ((err = runtime->hw_memset(subchn, v, dstoff, snd_pcm_lib_silence(subchn), size)) < 0)
+					return err;
+				snd_pcm_proc_write_silence(subchn, runtime->position, size);
+			} else {
+				if ((err = runtime->hw_memcpy(subchn, v, dstoff, bufs[v] + srcoff, size)) < 0)
+					return err;
+				snd_pcm_proc_write(subchn, runtime->position, bufs[v] + srcoff, size, 0);
+			}
+		}
+	} else {
+		/* default transfer behaviour */
+		int buf_size = runtime->buf.stream.queue_size / voices;
+		for (v = 0; v < voices; ++v) {
+			char *dest = runtime->dma_area->buf + (v * buf_size) + dstoff;
+			if (bufs[v] == NULL) {
+				memset(dest, snd_pcm_lib_silence(subchn), size);
+				snd_pcm_proc_write_silence(subchn, runtime->position, size);
+			} else {
+				if (copy_from_user(dest, bufs[v] + srcoff, size))
+					return -EFAULT;
+				snd_pcm_proc_write(subchn, runtime->position, bufs[v] + srcoff, size, 0);
+			}
 		}
-		snd_pcm_proc_write(subchn, runtime->position, bufs[v] + srcoff, size, 0);
 	}
 	return 0;
 }
@@ -1302,7 +1372,7 @@ long snd_pcm_lib_writev(snd_pcm_subchn_t *subchn, const struct iovec *vector,
 	snd_debug_check(runtime == NULL, -ENXIO);
 	snd_debug_check(runtime->dma_area == NULL, -EINVAL);
 	if (*runtime->status < SND_PCM_STATUS_PREPARED)
-		return -EINVAL;
+		return -EBADFD;
 
 	snd_debug_check(subchn->ffile == NULL, -ENXIO);
 	nonblock = !!(subchn->ffile->f_flags & O_NONBLOCK);
@@ -1324,9 +1394,12 @@ long snd_pcm_lib_writev(snd_pcm_subchn_t *subchn, const struct iovec *vector,
 			size_t len = vector[b].iov_len;
 			int ret;
 			if (runtime->mode == SND_PCM_MODE_BLOCK)
-				ret = snd_pcm_lib_write_block(subchn, buf, len, nonblock, snd_pcm_lib_write_transfer);
+				ret = snd_pcm_lib_write_block(subchn, buf, len, nonblock,
+							      snd_pcm_lib_write_transfer);
 			else if (runtime->mode == SND_PCM_MODE_STREAM)
-				ret = snd_pcm_lib_write_stream(subchn, buf, len, nonblock, snd_pcm_lib_write_transfer);
+				ret = snd_pcm_lib_write_stream(subchn, buf, len, nonblock,
+							       snd_pcm_lib_write_transfer,
+							       snd_pcm_lib_write_silence);
 			else
 				return -EIO;
 			if (ret < 0)
@@ -1352,13 +1425,16 @@ long snd_pcm_lib_writev(snd_pcm_subchn_t *subchn, const struct iovec *vector,
 					count = len;
 				else if (count != len)
 					return -EINVAL;
-				if (verify_area(VERIFY_READ, bufs[v], len))
+				if (bufs[v] != NULL && verify_area(VERIFY_READ, bufs[v], len))
 					return -EFAULT;
 			}
 			if (runtime->mode == SND_PCM_MODE_BLOCK)
-				ret = snd_pcm_lib_write_block(subchn, bufs, count * voices, nonblock, snd_pcm_lib_writev_transfer);
+				ret = snd_pcm_lib_write_block(subchn, bufs, count * voices, nonblock,
+							      snd_pcm_lib_writev_transfer);
 			else if (runtime->mode == SND_PCM_MODE_STREAM)
-				ret = snd_pcm_lib_write_stream(subchn, bufs, count * voices, nonblock, snd_pcm_lib_writev_transfer);
+				ret = snd_pcm_lib_write_stream(subchn, bufs, count * voices, nonblock,
+							       snd_pcm_lib_writev_transfer,
+							       snd_pcm_lib_writev_silence);
 			else
 				return -EIO;
 			if (ret < 0)
@@ -1409,7 +1485,7 @@ static long snd_pcm_lib_read_block(snd_pcm_subchn_t *subchn, void *dst, unsigned
 			if (runtime->start_mode == SND_PCM_START_DATA)
 				snd_pcm_kernel_capture_ioctl(subchn, SND_PCM_IOCTL_CHANNEL_GO, 0);
 			if (*runtime->status != SND_PCM_STATUS_RUNNING)
-				return result > 0 ? result : -EIO;
+				return result > 0 ? result : -EPIPE;
 			if (nonblock)
 				return result > 0 ? result : -EAGAIN;
 			timeout = interruptible_sleep_on_timeout(&runtime->sleep, 10 * HZ);
@@ -1426,7 +1502,7 @@ static long snd_pcm_lib_read_block(snd_pcm_subchn_t *subchn, void *dst, unsigned
 		    *runtime->status != SND_PCM_STATUS_RUNNING) {
 			spin_unlock_irqrestore(&runtime->lock, flags);
 			// snd_printd("capture read error (status = %i)\n", *runtime->status);
-			return result > 0 ? result : -EIO;
+			return result > 0 ? result : -EPIPE;
 		}
 		block = (*runtime->frag_tail)++;
 		*runtime->frag_tail %= runtime->frags;
@@ -1478,13 +1554,13 @@ static long snd_pcm_lib_read_stream(snd_pcm_subchn_t *subchn, void *dst, unsigne
 			if (runtime->stop_mode != SND_PCM_STOP_ROLLOVER) {
 				spin_unlock_irqrestore(&runtime->lock, flags);
 				snd_pcm_transfer_stop(subchn, SND_PCM_STATUS_OVERRUN);
-				return -EIO;
+				return -EPIPE;
 			}
 		}
 	} else {
 		if (*runtime->status != SND_PCM_STATUS_PREPARED) {
 			spin_unlock_irqrestore(&runtime->lock, flags);
-			return -EIO;
+			return -EPIPE;
 		}
 		if (runtime->start_mode == SND_PCM_START_DATA) {
 			spin_unlock_irqrestore(&runtime->lock, flags);
@@ -1518,7 +1594,7 @@ static long snd_pcm_lib_read_stream(snd_pcm_subchn_t *subchn, void *dst, unsigne
 			return result > 0 ? result : err;
 		spin_lock_irqsave(&runtime->lock, flags);
 		if (*runtime->status != SND_PCM_STATUS_RUNNING) {
-			result = result > 0 ? result : -EIO;
+			result = result > 0 ? result : -EPIPE;
 			break;
 		}
 		offset += size;
@@ -1529,7 +1605,7 @@ static long snd_pcm_lib_read_stream(snd_pcm_subchn_t *subchn, void *dst, unsigne
 			snd_pcm_lib_boundary_capture(runtime);
 	}
 	spin_unlock_irqrestore(&runtime->lock, flags);
-	return result;
+	return result == 0 ? -EAGAIN : result;
 }
 
 long snd_pcm_lib_read(snd_pcm_subchn_t *subchn, char *buf, long count)
@@ -1542,7 +1618,7 @@ long snd_pcm_lib_read(snd_pcm_subchn_t *subchn, char *buf, long count)
 	snd_debug_check(runtime == NULL, -ENXIO);
 	snd_debug_check(runtime->dma_area == NULL, -EINVAL);
 	if (*runtime->status < SND_PCM_STATUS_PREPARED)
-		return -EINVAL;
+		return -EBADFD;
 
 	snd_debug_check(subchn->ffile == NULL, -ENXIO);
 	nonblock = !!(subchn->ffile->f_flags & O_NONBLOCK);
@@ -1603,7 +1679,7 @@ long snd_pcm_lib_readv(snd_pcm_subchn_t *subchn, const struct iovec *vector,
 	snd_debug_check(runtime == NULL, -ENXIO);
 	snd_debug_check(runtime->dma_area == NULL, -EINVAL);
 	if (*runtime->status < SND_PCM_STATUS_PREPARED)
-		return -EINVAL;
+		return -EBADFD;
 
 	snd_debug_check(subchn->ffile == NULL, -ENXIO);
 	nonblock = !!(subchn->ffile->f_flags & O_NONBLOCK);
@@ -1653,7 +1729,7 @@ long snd_pcm_lib_readv(snd_pcm_subchn_t *subchn, const struct iovec *vector,
 					count = len;
 				else if (count != len)
 					return -EINVAL;
-				if (verify_area(VERIFY_WRITE, bufs[v], len))
+				if (bufs[v] != NULL && verify_area(VERIFY_WRITE, bufs[v], len))
 					return -EFAULT;
 			}
 			if (runtime->mode == SND_PCM_MODE_BLOCK)
