@@ -534,7 +534,7 @@ int __init alsa_sound_init(void)
 #ifdef CONFIG_DEVFS_FS
 	for (controlnum = 0; controlnum < snd_cards_limit; controlnum++) {
 		sprintf(controlname, "snd/controlC%d", controlnum);
-		devfs_register(NULL, controlname , 0, DEVFS_FL_TOLERANT,
+		devfs_register(NULL, controlname, 0, DEVFS_FL_DEFAULT,
 				snd_major, controlnum<<5, snd_device_mode | S_IFCHR,
 				0, 0, &snd_fops, NULL);
 	}
