@@ -566,7 +566,7 @@ static snd_pcm_hardware_t snd_emu10k1_playback =
 	SND_PCM_CHNINFO_BLOCK_TRANSFER |
 	SND_PCM_CHNINFO_MMAP_VALID,		/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000, /* supported RATES */
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000, /* supported RATES */
 	4000,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
@@ -943,7 +943,7 @@ static int snd_emu10k1_pcm_efx_set_voices_mask(snd_pcm_t * pcm,
 		if (uswitch->value.data32[0] & (1 << idx))
 			bits++;
 	}
-	if (bits != 1 || bits != 2 || bits != 4 || bits != 8 || bits != 16 || bits != 32)
+	if (bits != 1 && bits != 2 && bits != 4 && bits != 8 && bits != 16 && bits != 32)
 		return -EINVAL;
 	spin_lock_irqsave(&emu->reg_lock, flags);
 	emu->efx_voices_mask = uswitch->value.data32[0];
