@@ -46,6 +46,7 @@ static struct {
 	{	0x43525913,	"Cirrus Logic CS4297A"	},
 	{	0x42525923,	"Cirrus Logic CS4294"	},
 	{	0x43525931,	"Cirrus Logic CS4299"	},
+	{	0x43525933,	"Cirrus Logic CS4299+"	}, // some new revision?
 	{	0x4e534331,	"National Semiconductor LM4549"},
 	{	0x54524108,	"TriTech TR28028"	}, // added by xin jin [07/09/99]
 	{	0x574D4C00,	"Wolfson WM9704"	}, // added by xin jin [07/09/99]
@@ -93,6 +94,59 @@ static char *snd_ac97_stereo_enhancements[] =
   /*  31 */ "Reserved 31"
 };
 
+/*
+ *  I/O routines, filter some registers for buggy codecs
+ */
+
+static int snd_ac97_valid_reg(ac97_t *ac97, unsigned short reg)
+{
+	switch (ac97->id) {
+	case 0x414B4D00:	/* AK4540 */
+		if (reg <= 0x1c || reg == 0x20 || reg == 0x26 || reg >= 0x7c)
+			return 1;
+		return 0;
+	case 0x41445340:	/* AD1881 */
+		if (reg >= 0x3a && reg <= 0x59)
+			return 0;
+		return 1;
+	}
+	return 1;
+}
+
+void snd_ac97_write(ac97_t *ac97, unsigned short reg, unsigned short value)
+{
+	if (!snd_ac97_valid_reg(ac97, reg))
+		return;
+	ac97->write(ac97->private_data, reg, value);
+}
+
+unsigned short snd_ac97_read(ac97_t *ac97, unsigned short reg)
+{
+	if (!snd_ac97_valid_reg(ac97, reg))
+		return 0;
+	return ac97->read(ac97->private_data, reg);
+}
+
+void snd_ac97_write_lock(ac97_t *ac97, unsigned short reg, unsigned short value)
+{
+	unsigned long flags;
+
+	spin_lock_irqsave(&ac97->reg_lock, flags);
+	snd_ac97_write(ac97, reg, value);
+	spin_unlock_irqrestore(&ac97->reg_lock, flags);
+}
+
+unsigned short snd_ac97_read_lock(ac97_t *ac97, unsigned short reg)
+{
+	unsigned long flags;
+	unsigned short result;
+
+	spin_lock_irqsave(&ac97->reg_lock, flags);
+	result = snd_ac97_read(ac97, reg);
+	spin_unlock_irqrestore(&ac97->reg_lock, flags);
+	return result;
+}
+
 /*
  *
  */
@@ -160,7 +214,7 @@ static int snd_ac97_mux(snd_kmixer_element_t *element, int w_flag, snd_kmixer_el
 		change = elements[0] != left || elements[1] != right;
 		nleft = snd_ac97_get_recsrc(ac97, elements[0]);
 		nright = snd_ac97_get_recsrc(ac97, elements[1]);
-		ac97->write(ac97->private_data, AC97_REC_SEL,
+		snd_ac97_write(ac97, AC97_REC_SEL,
 			(ac97->regs[AC97_REC_SEL] = (nleft << 8) | nright));
 	}
 	spin_unlock_irqrestore(&ac97->reg_lock, flags);
@@ -185,7 +239,7 @@ static int snd_ac97_mono_out_mux(snd_kmixer_element_t *element, int w_flag, snd_
 		ac97->regs[AC97_GENERAL_PURPOSE] &= ~0x0200;
 		if (*melement == ac97->me_mux2_mic)
 			ac97->regs[AC97_GENERAL_PURPOSE] |= 0x0200;
-		ac97->write(ac97->private_data, AC97_GENERAL_PURPOSE, ac97->regs[AC97_GENERAL_PURPOSE]);
+		snd_ac97_write(ac97, AC97_GENERAL_PURPOSE, ac97->regs[AC97_GENERAL_PURPOSE]);
 	}
 	spin_unlock_irqrestore(&ac97->reg_lock, flags);
 	return change;
@@ -206,7 +260,7 @@ static int snd_ac97_mute(snd_kmixer_element_t *element, int w_flag, int *value,
 	} else {
 		change = val != *value;
 		val = (ac97->regs[reg] & 0x7fff) | (*value ? 0 : 0x8000);
-		ac97->write(ac97->private_data, reg, ac97->regs[reg] = val);
+		snd_ac97_write(ac97, reg, ac97->regs[reg] = val);
 	}
 	spin_unlock_irqrestore(&ac97->reg_lock, flags);
 	return change;
@@ -248,7 +302,7 @@ static int snd_ac97_mute_center(snd_kmixer_element_t *element, int w_flag, int *
 		ac97->regs[AC97_CENTER_LFE_MASTER] &= 0xff7f;
 		if (!snd_mixer_get_bit(value, 0))
 			ac97->regs[AC97_CENTER_LFE_MASTER] |= 0x0080;
-		ac97->write(ac97->private_data, AC97_CENTER_LFE_MASTER, ac97->regs[AC97_CENTER_LFE_MASTER]);
+		snd_ac97_write(ac97, AC97_CENTER_LFE_MASTER, ac97->regs[AC97_CENTER_LFE_MASTER]);
 	}
 	spin_unlock_irqrestore(&ac97->reg_lock, flags);
 	return change;
@@ -270,7 +324,7 @@ static int snd_ac97_mute_lfe(snd_kmixer_element_t *element, int w_flag, int *val
 		ac97->regs[AC97_CENTER_LFE_MASTER] &= 0x7fff;
 		if (!snd_mixer_get_bit(value, 0))
 			ac97->regs[AC97_CENTER_LFE_MASTER] |= 0x8000;
-		ac97->write(ac97->private_data, AC97_CENTER_LFE_MASTER, ac97->regs[AC97_CENTER_LFE_MASTER]);
+		snd_ac97_write(ac97, AC97_CENTER_LFE_MASTER, ac97->regs[AC97_CENTER_LFE_MASTER]);
 	}
 	spin_unlock_irqrestore(&ac97->reg_lock, flags);
 	return change;
@@ -298,7 +352,7 @@ static int snd_ac97_mute_surround(snd_kmixer_element_t *element, int w_flag, uns
 		ac97->regs[AC97_SURROUND_MASTER] &= 0xff7f;
 		if (!snd_mixer_get_bit(value, 1))
 			ac97->regs[AC97_SURROUND_MASTER] |= 0x0080;
-		ac97->write(ac97->private_data, AC97_SURROUND_MASTER, ac97->regs[AC97_SURROUND_MASTER]);
+		snd_ac97_write(ac97, AC97_SURROUND_MASTER, ac97->regs[AC97_SURROUND_MASTER]);
 	}
 	spin_unlock_irqrestore(&ac97->reg_lock, flags);
 	return change;
@@ -346,7 +400,7 @@ static int snd_ac97_tone_control(snd_kmixer_element_t *element, int w_flag,
 			}
 			ac97->treble = treble;
 		}
-		ac97->write(ac97->private_data, AC97_MASTER_TONE, ac97->regs[AC97_MASTER_TONE]);
+		snd_ac97_write(ac97, AC97_MASTER_TONE, ac97->regs[AC97_MASTER_TONE]);
 	}
 	spin_unlock_irqrestore(&ac97->reg_lock, flags);
 	return change;	
@@ -374,7 +428,7 @@ static int snd_ac97_volume_stereo(snd_kmixer_element_t *element, int w_flag, int
 		ac97->regs[reg] &= ~((max << 8) | max);
 		ac97->regs[reg] |= !invert ? ((max - volume[0]) << 8) | (max - volume[1]) :
 					     (volume[0] << 8) | volume[1];
-		ac97->write(ac97->private_data, reg, ac97->regs[reg]);
+		snd_ac97_write(ac97, reg, ac97->regs[reg]);
 	}
 	spin_unlock_irqrestore(&ac97->reg_lock, flags);
 	return change;
@@ -397,7 +451,7 @@ static int snd_ac97_volume_mono(snd_kmixer_element_t *element, int w_flag, int *
 		change = volume[0] != val;
 		ac97->regs[reg] &= ~max;
 		ac97->regs[reg] |= !invert ? max - volume[0] : volume[0];
-		ac97->write(ac97->private_data, reg, ac97->regs[reg]);
+		snd_ac97_write(ac97, reg, ac97->regs[reg]);
 	}
 	spin_unlock_irqrestore(&ac97->reg_lock, flags);
 	return change;
@@ -439,7 +493,7 @@ static int snd_ac97_volume_pc_beep(snd_kmixer_element_t *element, int w_flag, in
 		change = volume[0] != val;
 		ac97->regs[AC97_PC_BEEP] &= ~(15 << 1);
 		ac97->regs[AC97_PC_BEEP] |= (15 - volume[0]) << 1;
-		ac97->write(ac97->private_data, AC97_PC_BEEP, ac97->regs[AC97_PC_BEEP]);
+		snd_ac97_write(ac97, AC97_PC_BEEP, ac97->regs[AC97_PC_BEEP]);
 	}
 	spin_unlock_irqrestore(&ac97->reg_lock, flags);
 	return change;
@@ -460,7 +514,7 @@ static int snd_ac97_mic_boost_volume(snd_kmixer_element_t *element, int w_flag,
 		change = volume[0] != val;
 		ac97->regs[AC97_MIC] &= ~(1 << 6);
 		ac97->regs[AC97_MIC] |= volume[0] << 6;
-		ac97->write(ac97->private_data, AC97_MIC, ac97->regs[AC97_MIC]);
+		snd_ac97_write(ac97, AC97_MIC, ac97->regs[AC97_MIC]);
 	}
 	spin_unlock_irqrestore(&ac97->reg_lock, flags);
 	return change;
@@ -481,7 +535,7 @@ static int snd_ac97_volume_lfe(snd_kmixer_element_t *element, int w_flag, int *v
 		change = volume[0] != val;
 		ac97->regs[AC97_CENTER_LFE_MASTER] &= ~(ac97->max_lfe << 8);
 		ac97->regs[AC97_CENTER_LFE_MASTER] |= volume[0] << 8;
-		ac97->write(ac97->private_data, AC97_CENTER_LFE_MASTER, ac97->regs[AC97_CENTER_LFE_MASTER]);
+		snd_ac97_write(ac97, AC97_CENTER_LFE_MASTER, ac97->regs[AC97_CENTER_LFE_MASTER]);
 	}
 	spin_unlock_irqrestore(&ac97->reg_lock, flags);
 	return change;
@@ -540,8 +594,8 @@ static int snd_ac97_3d(snd_kmixer_element_t *element, int w_flag, struct snd_mix
 			ac97->regs[AC97_3D_CONTROL] &= ~(((ac97->max_3d << 8) | ac97->max_3d) << ac97->shift_3d);
 			ac97->regs[AC97_3D_CONTROL] |= ((effect1->center << 8) | effect1->depth) << ac97->shift_3d;
 		}
-		ac97->write(ac97->private_data, AC97_3D_CONTROL, ac97->regs[AC97_3D_CONTROL]);
-		ac97->write(ac97->private_data, AC97_GENERAL_PURPOSE, ac97->regs[AC97_GENERAL_PURPOSE]);
+		snd_ac97_write(ac97, AC97_3D_CONTROL, ac97->regs[AC97_3D_CONTROL]);
+		snd_ac97_write(ac97, AC97_GENERAL_PURPOSE, ac97->regs[AC97_GENERAL_PURPOSE]);
 	}
 	spin_unlock_irqrestore(&ac97->reg_lock, flags);	
 	return change;
@@ -585,8 +639,8 @@ static int snd_ac97_set_switch(snd_kmixer_t * mixer,
 	spin_lock_irqsave(&ac97->reg_lock, flags);
 	val = ac97->regs[reg];
 	change = (val & mask) != val1 ? 1 : 0;
-	ac97->regs[reg] = (val & ~mask) | val1;
-	ac97->write(ac97->private_data, reg, val);
+	val = (val & ~mask) | val1;
+	snd_ac97_write(ac97, reg, ac97->regs[reg] = val);
 	spin_unlock_irqrestore(&ac97->reg_lock, flags);
 	return change;
 }
@@ -681,27 +735,27 @@ static int snd_ac97_try_volume_mix(ac97_t * ac97, int reg)
 		return ac97->caps & 0x01 ? 1 : 0;
 	case AC97_3D_CONTROL:
 		if (ac97->caps & 0x7c00) {
-			val = ac97->read(ac97->private_data, reg);
+			val = snd_ac97_read(ac97, reg);
 			/* if nonzero - fixed and we can't set it */
 			return val == 0;
 		}
 		return 0;
 	case AC97_CENTER_LFE_MASTER:	/* center */
-		val = ac97->read(ac97->private_data, AC97_EXTENDED_STATUS);
+		val = snd_ac97_read(ac97, AC97_EXTENDED_STATUS);
 		return val != 0xffff && (val & 0x40) ? 1 : 0;
 	case AC97_CENTER_LFE_MASTER+1:	/* lfe */
-		val = ac97->read(ac97->private_data, AC97_EXTENDED_STATUS);
+		val = snd_ac97_read(ac97, AC97_EXTENDED_STATUS);
 		return val != 0xffff && (val & 0x100) ? 1 : 0;		
 	case AC97_SURROUND_MASTER: /* read extended audio status control */
-		val = ac97->read(ac97->private_data, AC97_EXTENDED_STATUS);
+		val = snd_ac97_read(ac97, AC97_EXTENDED_STATUS);
 		return val != 0xffff && (val & 0x80) ? 1 : 0;
 	}
-	val = ac97->read(ac97->private_data, reg);
+	val = snd_ac97_read(ac97, reg);
 	if (!(val & 0x8000)) {
 		/* nothing seems to be here - mute flag isn't set */
 		/* try another test */
-		ac97->write(ac97->private_data, reg, val | 0x8000);
-		val = ac97->read(ac97->private_data, reg);
+		snd_ac97_write(ac97, reg, val | 0x8000);
+		val = snd_ac97_read(ac97, reg);
 		if (!(val & 0x8000))
 			return 0;	/* nothing here */
 	}
@@ -714,11 +768,11 @@ static int snd_ac97_try_switch(ac97_t * ac97, snd_kswitch_t * kswitch)
 
 	reg = kswitch->private_value >> 16;
 	mask = kswitch->private_value & 0xffff;
-	orig = ac97->read(ac97->private_data, reg);
+	orig = snd_ac97_read(ac97, reg);
 	val = orig ^ mask;
-	ac97->write(ac97->private_data, reg, val);
-	res = ac97->read(ac97->private_data, reg);
-	ac97->write(ac97->private_data, reg, orig);
+	snd_ac97_write(ac97, reg, val);
+	res = snd_ac97_read(ac97, reg);
+	snd_ac97_write(ac97, reg, orig);
 	return res == val;
 }
 
@@ -728,13 +782,13 @@ static void snd_ac97_change_volume_params1(ac97_t * ac97, int reg, unsigned char
 
 	*max = 63;
 	val = 0x8000 | 0x0020;
-	ac97->write(ac97->private_data, reg, val);
-	val1 = ac97->read(ac97->private_data, reg);
+	snd_ac97_write(ac97, reg, val);
+	val1 = snd_ac97_read(ac97, reg);
 	if (val != val1) {
 		*max = 31;
 	}
 	/* reset volume to zero */
-	ac97->write(ac97->private_data, reg, 0x8000);
+	snd_ac97_write(ac97, reg, 0x8000);
 }
 
 static void snd_ac97_change_volume_params2(ac97_t * ac97, int reg, unsigned char *max)
@@ -743,13 +797,13 @@ static void snd_ac97_change_volume_params2(ac97_t * ac97, int reg, unsigned char
 
 	*max = 63;
 	val = 0x8000 | 0x2000;
-	ac97->write(ac97->private_data, reg, val);
-	val1 = ac97->read(ac97->private_data, reg);
+	snd_ac97_write(ac97, reg, val);
+	val1 = snd_ac97_read(ac97, reg);
 	if (val != val1) {
 		*max = 31;
 	}
 	/* reset volume to zero */
-	ac97->write(ac97->private_data, reg, 0x8080);
+	snd_ac97_write(ac97, reg, 0x8080);
 }
 
 static void snd_ac97_change_3d_params(ac97_t * ac97)
@@ -759,24 +813,24 @@ static void snd_ac97_change_3d_params(ac97_t * ac97)
 	if (ac97->id == 0x83847608) {	// Sigmatel 9708
 		ac97->max_3d = 3;
 		ac97->shift_3d = 0;	// not used.
-		ac97->write(ac97->private_data, AC97_3D_CONTROL, 0x000f);
+		snd_ac97_write(ac97, AC97_3D_CONTROL, 0x000f);
 	} else if (ac97->id == 0x83847609) {	// Sigmatel 9721/23
 		ac97->max_3d = 3;
 		// manual says that DP0-1 bits are valid,
 		// but it doesn't seem true (maybe some early revisions?)
 		ac97->shift_3d = 2;
-		ac97->write(ac97->private_data, AC97_3D_CONTROL, 0x0000);
+		snd_ac97_write(ac97, AC97_3D_CONTROL, 0x0000);
 	} else {
 		ac97->max_3d = 15;
 		ac97->shift_3d = 0;
 		val = 0x0707;
-		ac97->write(ac97->private_data, AC97_3D_CONTROL, val);
-		val = ac97->read(ac97->private_data, AC97_3D_CONTROL);
+		snd_ac97_write(ac97, AC97_3D_CONTROL, val);
+		val = snd_ac97_read(ac97, AC97_3D_CONTROL);
 		if (val == 0x0606) {
 			ac97->max_3d = 7;
 			ac97->shift_3d = 1;
 		}
-		ac97->write(ac97->private_data, AC97_3D_CONTROL, 0x0f0f);
+		snd_ac97_write(ac97, AC97_3D_CONTROL, 0x0f0f);
 	}
 }
 
@@ -1833,21 +1887,21 @@ int snd_ac97_mixer(snd_card_t * card, int device,
 	mixer->private_data = ac97;
 	mixer->private_free = snd_ac97_mixer_free;
 	strcpy(mixer->name, "AC97 generic");
-	ac97->write(ac97->private_data, AC97_RESET, 0);		/* reset to defaults */
+	snd_ac97_write(ac97, AC97_RESET, 0);		/* reset to defaults */
 	udelay(100);
-	ac97->id = ac97->read(ac97->private_data, AC97_VENDOR_ID1) << 16;
-	ac97->id |= ac97->read(ac97->private_data, AC97_VENDOR_ID2);
+	ac97->id = snd_ac97_read(ac97, AC97_VENDOR_ID1) << 16;
+	ac97->id |= snd_ac97_read(ac97, AC97_VENDOR_ID2);
 	if (ac97->id == 0 || ac97->id == -1) {
 		snd_printk("The AC'97 access is not valid, removing mixer.\n");
 		snd_device_free(card, mixer);
 		return -EIO;
 	}
-	ac97->caps = ac97->read(ac97->private_data, AC97_RESET);
-	ac97->write(ac97->private_data, AC97_POWERDOWN, 0);	/* nothing should be in powerdown mode */
-	ac97->write(ac97->private_data, AC97_RESET, 0);		/* reset to defaults */
+	ac97->caps = snd_ac97_read(ac97, AC97_RESET);
+	snd_ac97_write(ac97, AC97_POWERDOWN, 0);	/* nothing should be in powerdown mode */
+	snd_ac97_write(ac97, AC97_RESET, 0);		/* reset to defaults */
 	udelay(100);
-	ac97->write(ac97->private_data, AC97_POWERDOWN, 0);	/* nothing should be in powerdown mode */
-	ac97->write(ac97->private_data, AC97_GENERAL_PURPOSE, 0);
+	snd_ac97_write(ac97, AC97_POWERDOWN, 0);	/* nothing should be in powerdown mode */
+	snd_ac97_write(ac97, AC97_GENERAL_PURPOSE, 0);
 	if (ac97->init)
 		ac97->init(ac97->private_data);
 	if (snd_ac97_mixer_build(mixer, ac97, pcm_count, pcm_dev) < 0) {
@@ -1859,7 +1913,7 @@ int snd_ac97_mixer(snd_card_t * card, int device,
 			snd_mixer_switch_new(mixer, &snd_ac97_switches[idx], ac97);
 	}
 	if (ac97->id == 0x83847608 || ac97->id == 0x83847609) {
-		ac97->regs[AC97_SIGMATEL_ANALOG] = ac97->read(ac97->private_data, AC97_SIGMATEL_ANALOG);
+		ac97->regs[AC97_SIGMATEL_ANALOG] = snd_ac97_read(ac97, AC97_SIGMATEL_ANALOG);
 		for (idx = 0; idx < AC97_SIGMATEL_SWITCHES; idx++)
 			if (snd_ac97_try_switch(ac97, &snd_ac97_sigmatel_switches[idx]))
 				snd_mixer_switch_new(mixer, &snd_ac97_sigmatel_switches[idx], ac97);
@@ -1905,7 +1959,7 @@ int snd_ac97_mixer(snd_card_t * card, int device,
 
 static void snd_ac97_proc_read(snd_info_buffer_t * buffer, void *private_data)
 {
-	unsigned short val;
+	unsigned short val, ext;
 	ac97_t *ac97 = snd_magic_cast(ac97_t, private_data, );
 
 	if (ac97->rev_is_not_rev) {
@@ -1935,9 +1989,9 @@ static void snd_ac97_proc_read(snd_info_buffer_t * buffer, void *private_data)
 	snd_iprintf(buffer, "3D enhancement   : %s\n",
 		snd_ac97_stereo_enhancements[(ac97->caps >> 10) & 0x1f]);
 	snd_iprintf(buffer, "\nCurrent setup\n");
-	val = ac97->read(ac97->private_data, AC97_MIC);
-	snd_iprintf(buffer, "MIC gain         : %s\n", val & 0x0040 ? "+20dB" : "+0dB");
-	val = ac97->read(ac97->private_data, AC97_GENERAL_PURPOSE);
+	val = snd_ac97_read_lock(ac97, AC97_MIC);
+	snd_iprintf(buffer, "MIC gain         : %s [%s]\n", val & 0x0040 ? "+20dB" : "+0dB", ac97->regs[AC97_MIC] & 0x0040 ? "+20dB" : "+0dB");
+	val = snd_ac97_read_lock(ac97, AC97_GENERAL_PURPOSE);
 	snd_iprintf(buffer, "POP path         : %s 3D\n"
 		    "Sim. stereo      : %s\n"
 		    "3D enhancement   : %s\n"
@@ -1952,6 +2006,60 @@ static void snd_ac97_proc_read(snd_info_buffer_t * buffer, void *private_data)
 		    val & 0x0200 ? "MIC" : "MIX",
 		    val & 0x0100 ? "MIC2" : "MIC1",
 		    val & 0x0080 ? "on" : "off");
+	val = ext = snd_ac97_read_lock(ac97, AC97_EXTENDED_ID);
+	if (val == 0)
+		return;
+	snd_iprintf(buffer, "Extended ID      : codec=%i%s%s%s%s%s%s%s\n",
+			(val >> 14) & 3,
+			val & 0x0200 ? " AMAP" : "",
+			val & 0x0100 ? " LDAC" : "",
+			val & 0x0080 ? " SDAC" : "",
+			val & 0x0040 ? " CDAC" : "",
+			val & 0x0008 ? " VRM" : "",
+			val & 0x0002 ? " DRA" : "",
+			val & 0x0001 ? " VRA" : "");
+	val = snd_ac97_read_lock(ac97, AC97_EXTENDED_STATUS);
+	snd_iprintf(buffer, "Extended status  :%s%s%s%s%s%s%s%s%s%s%s\n",
+			val & 0x4000 ? " PRL" : "",
+			val & 0x2000 ? " PRK" : "",
+			val & 0x1000 ? " PRJ" : "",
+			val & 0x0800 ? " PRI" : "",
+			val & 0x0200 ? " MADC" : "",
+			val & 0x0100 ? " LDAC" : "",
+			val & 0x0080 ? " SDAC" : "",
+			val & 0x0040 ? " CDAC" : "",
+			val & 0x0008 ? " VRM" : "",
+			val & 0x0002 ? " DRA" : "",
+			val & 0x0001 ? " VRA" : "");
+	if (ext & 1) {	/* VRA */
+		val = snd_ac97_read_lock(ac97, AC97_PCM_FRONT_DAC_RATE);
+		snd_iprintf(buffer, "PCM front DAC    : %iHz\n", val);
+		if (ext & 0x0080) {
+			val = snd_ac97_read_lock(ac97, AC97_PCM_SURR_DAC_RATE);
+			snd_iprintf(buffer, "PCM Surr DAC     : %iHz\n", val);
+		}
+		if (ext & 0x0100) {
+			val = snd_ac97_read_lock(ac97, AC97_PCM_LFE_DAC_RATE);
+			snd_iprintf(buffer, "PCM LFE DAC      : %iHz\n", val);
+		}
+		val = snd_ac97_read_lock(ac97, AC97_PCM_LR_ADC_RATE);
+		snd_iprintf(buffer, "PCM ADC          : %iHz\n", val);
+	}
+	if (ext & 0x0008) {
+		val = snd_ac97_read_lock(ac97, AC97_PCM_MIC_ADC_RATE);
+		snd_iprintf(buffer, "PCM MIC ADC      : %iHz\n", val);
+	}
+}
+
+static void snd_ac97_proc_regs_read(snd_info_buffer_t * buffer, void *private_data)
+{
+	int reg, val;
+	ac97_t *ac97 = snd_magic_cast(ac97_t, private_data, );
+
+	for (reg = 0; reg < 0x80; reg += 2) {
+		val = snd_ac97_read_lock(ac97, reg);
+		snd_iprintf(buffer, "%02x = %04x\n", reg, val);
+	}
 }
 
 #if 0
@@ -1961,8 +2069,8 @@ static void snd_ac97_outm(ac97_t * ac97, unsigned short reg, unsigned short mask
 	unsigned short old;
 
 	spin_lock_irqsave(&ac97->reg_lock, flags);
-	old = ac97->read(ac97->private_data, reg);
-	ac97->write(ac97->private_data, reg, (old & mask) | val);
+	old = snd_ac97_read(ac97, reg);
+	snd_ac97_write(ac97, reg, (old & mask) | val);
 	spin_unlock_irqrestore(&ac97->reg_lock, flags);
 }
 #endif
@@ -1982,18 +2090,31 @@ static void snd_ac97_proc_init(snd_card_t * card, ac97_t * ac97)
 		}
 	}
 	ac97->proc_entry = entry;
+	if ((entry = snd_info_create_entry(card, "ac97regs")) != NULL) {
+		entry->private_data = ac97;
+		entry->mode = S_IFREG | S_IRUGO | S_IWUSR;
+		entry->t.text.read_size = 1024;
+		entry->t.text.read = snd_ac97_proc_regs_read;
+		if (snd_info_register(entry) < 0) {
+			snd_info_free_entry(entry);
+			entry = NULL;
+		}
+	}
+	ac97->proc_regs_entry = entry;
 }
 
 static void snd_ac97_proc_done(ac97_t * ac97)
 {
+	if (ac97->proc_regs_entry) {
+		snd_info_unregister(ac97->proc_regs_entry);
+		ac97->proc_regs_entry = NULL;
+	}
 	if (ac97->proc_entry) {
 		snd_info_unregister(ac97->proc_entry);
 		ac97->proc_entry = NULL;
 	}
 }
 
-EXPORT_SYMBOL(snd_ac97_mixer);
-
 /*
  *  Chip specific initialization
  */
@@ -2002,12 +2123,12 @@ static int snd_ac97_Sig8384_7609(ac97_t * ac97)
 {
 	// id1=0x8384, id2=0x7609
 	if (ac97->id == 0x83847609) {
-		if (ac97->read(ac97->private_data, AC97_SIGMATEL_ANALOG) == 0) {
+		if (snd_ac97_read(ac97, AC97_SIGMATEL_ANALOG) == 0) {
 			// patch for SigmaTel
-			ac97->write(ac97->private_data, AC97_SIGMATEL_CIC1, 0xabba);
-			ac97->write(ac97->private_data, AC97_SIGMATEL_CIC2, 0x4000);
-			ac97->write(ac97->private_data, AC97_SIGMATEL_BIAS1, 0xabba);
-			ac97->write(ac97->private_data, AC97_SIGMATEL_BIAS2, 0x0002);
+			snd_ac97_write(ac97, AC97_SIGMATEL_CIC1, 0xabba);
+			snd_ac97_write(ac97, AC97_SIGMATEL_CIC2, 0x4000);
+			snd_ac97_write(ac97, AC97_SIGMATEL_BIAS1, 0xabba);
+			snd_ac97_write(ac97, AC97_SIGMATEL_BIAS2, 0x0002);
 		}
 		return 1;
 	}
@@ -2020,18 +2141,18 @@ static int snd_ac97_Sig8384_7608(ac97_t * ac97)
 	unsigned int	codec72, codec6c;
 
 	if (ac97->id == 0x83847608) {
-		codec72 = ac97->read(ac97->private_data, AC97_SIGMATEL_BIAS2) & 0x8000;
-		codec6c = ac97->read(ac97->private_data, AC97_SIGMATEL_ANALOG);
+		codec72 = snd_ac97_read(ac97, AC97_SIGMATEL_BIAS2) & 0x8000;
+		codec6c = snd_ac97_read(ac97, AC97_SIGMATEL_ANALOG);
 
 		if ((codec72==0) && (codec6c==0)) {
-			ac97->write(ac97->private_data, AC97_SIGMATEL_CIC1, 0xabba);
-			ac97->write(ac97->private_data, AC97_SIGMATEL_CIC2, 0x1000);
-			ac97->write(ac97->private_data, AC97_SIGMATEL_BIAS1, 0xabba);
-			ac97->write(ac97->private_data, AC97_SIGMATEL_BIAS2, 0x0007);
+			snd_ac97_write(ac97, AC97_SIGMATEL_CIC1, 0xabba);
+			snd_ac97_write(ac97, AC97_SIGMATEL_CIC2, 0x1000);
+			snd_ac97_write(ac97, AC97_SIGMATEL_BIAS1, 0xabba);
+			snd_ac97_write(ac97, AC97_SIGMATEL_BIAS2, 0x0007);
 		} else if ((codec72==0x8000) && (codec6c==0)) {
-			ac97->write(ac97->private_data, AC97_SIGMATEL_CIC1, 0xabba);
-			ac97->write(ac97->private_data, AC97_SIGMATEL_CIC2, 0x1001);
-			ac97->write(ac97->private_data, AC97_SIGMATEL_DAC2INVERT, 0x0008);
+			snd_ac97_write(ac97, AC97_SIGMATEL_CIC1, 0xabba);
+			snd_ac97_write(ac97, AC97_SIGMATEL_CIC2, 0x1001);
+			snd_ac97_write(ac97, AC97_SIGMATEL_DAC2INVERT, 0x0008);
 		} else if ((codec72==0x8000) && (codec6c==0x0080)) {
 			/* nothing */
 		}
@@ -2045,16 +2166,16 @@ static int snd_ac97_Wolfson574D_4C0X(ac97_t * ac97)
 {
 	// detect the Wolfson codec
 	if ((ac97->id & 0xFFFFFF00) == 0x574D4C00) {
-		ac97->write(ac97->private_data, 0x72, 0x0808);
-		ac97->write(ac97->private_data, 0x74, 0x0808);
+		snd_ac97_write(ac97, 0x72, 0x0808);
+		snd_ac97_write(ac97, 0x74, 0x0808);
 
 		// patch for DVD noise
-		ac97->write(ac97->private_data, 0x5a, 0x0200);
+		snd_ac97_write(ac97, 0x5a, 0x0200);
 
 		// init vol as PCM vol
-		ac97->write(ac97->private_data, 0x70, ac97->read(ac97->private_data, AC97_PCM));
+		snd_ac97_write(ac97, 0x70, snd_ac97_read(ac97, AC97_PCM));
 
-		ac97->write(ac97->private_data, AC97_SURROUND_MASTER, 0x0000);
+		snd_ac97_write(ac97, AC97_SURROUND_MASTER, 0x0000);
 		return 1;
 	}
 	return 0;
@@ -2065,10 +2186,10 @@ static int snd_ac97_TriTech5452_4108(ac97_t * ac97)
 {
 	// detect the TriTech 5452 and 4108
 	if (ac97->id == 0x54524108) {
-		ac97->write(ac97->private_data, 0x26, 0x0300);
-		ac97->write(ac97->private_data, 0x26, 0x0000);
-		ac97->write(ac97->private_data, AC97_SURROUND_MASTER, 0x0000);
-		ac97->write(ac97->private_data, AC97_RESERVED_3A, 0x0000);
+		snd_ac97_write(ac97, 0x26, 0x0300);
+		snd_ac97_write(ac97, 0x26, 0x0000);
+		snd_ac97_write(ac97, AC97_SURROUND_MASTER, 0x0000);
+		snd_ac97_write(ac97, AC97_RESERVED_3A, 0x0000);
 		return 1;
 	}
 	return 0;
@@ -2086,6 +2207,16 @@ static void snd_ac97_chip_init(ac97_t * ac97)
 		return;
 }
 
+/*
+ *  Exported symbols
+ */
+
+EXPORT_SYMBOL(snd_ac97_write);
+EXPORT_SYMBOL(snd_ac97_read);
+EXPORT_SYMBOL(snd_ac97_write_lock);
+EXPORT_SYMBOL(snd_ac97_read_lock);
+EXPORT_SYMBOL(snd_ac97_mixer);
+
 /*
  *  INIT part
  */
