@@ -595,7 +595,7 @@ static snd_pcm_hardware_t snd_gf1_pcm_playback =
 	SND_PCM_CHNINFO_BLOCK,
 	SND_PCM_FMT_S8 | SND_PCM_FMT_U8 |
 	SND_PCM_FMT_S16_LE | SND_PCM_FMT_U16_LE,	/* hardware formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_44100,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_44100,
 	5510,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
@@ -617,7 +617,7 @@ static snd_pcm_hardware_t snd_gf1_pcm_capture =
 	SND_PCM_CHNINFO_BLOCK | SND_PCM_CHNINFO_INTERLEAVE |
 	SND_PCM_CHNINFO_MMAP_VALID,
 	SND_PCM_FMT_S8 | SND_PCM_FMT_U8,	/* hardware formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_44100,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_44100,
 	5510,			/* min. rate */
 	44100,			/* max. rate */
 	1,			/* min. voices */
