@@ -384,7 +384,7 @@ static snd_pcm_hardware_t snd_ad1816a_playback = {
 	/* formats */
 	SND_PCM_FMT_MU_LAW | SND_PCM_FMT_A_LAW |
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE | SND_PCM_FMT_S16_BE,
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000,
 	4000,			/* min rate */
 	55200,			/* max rate */
 	1,			/* min voices */
@@ -408,7 +408,7 @@ static snd_pcm_hardware_t snd_ad1816a_capture = {
 	/* formats */
 	SND_PCM_FMT_MU_LAW | SND_PCM_FMT_A_LAW |
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE | SND_PCM_FMT_S16_BE,
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000,
 	4000,			/* min rate */
 	55200,			/* max rate */
 	1,			/* min voices */
