@@ -34,6 +34,10 @@
  *  constants
  */
 
+#if 0
+#define ACCEPT_VALID		/* REQUIRED ONLY FOR OSS EMULATION */
+#endif
+
 #define CS461X_BA0_SIZE		0x2000
 #define CS461X_BA1_DATA0_SIZE	0x3000
 #define CS461X_BA1_DATA1_SIZE	0x3800
@@ -932,10 +936,13 @@ void snd_cs461x_interrupt(cs461x_t *codec)
 static snd_pcm_hardware_t snd_cs461x_playback =
 {
 	SND_PCM_CHNINFO_MMAP |
+#ifdef ACCEPT_VALID
+	SND_PCM_CHNINFO_VALID |		/* NOT TRUE!!! OSS REQUIRES IT */
+#endif
 	SND_PCM_CHNINFO_BLOCK | SND_PCM_CHNINFO_INTERLEAVE |
 	SND_PCM_CHNINFO_BLOCK_TRANSFER,		/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* hardware formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000,
 	5500,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
@@ -954,10 +961,13 @@ static snd_pcm_hardware_t snd_cs461x_playback =
 static snd_pcm_hardware_t snd_cs461x_capture =
 {
 	SND_PCM_CHNINFO_MMAP |
+#ifdef ACCEPT_VALID
+	SND_PCM_CHNINFO_VALID |		/* NOT TRUE!!! OSS REQUIRES IT */
+#endif
 	SND_PCM_CHNINFO_BLOCK | SND_PCM_CHNINFO_INTERLEAVE |
 	SND_PCM_CHNINFO_BLOCK_TRANSFER,		/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* hardware formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000,
 	5500,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
@@ -1234,13 +1244,9 @@ int snd_cs461x_midi(cs461x_t * codec, int device, snd_rawmidi_t **rrawmidi)
 	int err;
 
 	*rrawmidi = NULL;
-	if ((err = snd_rawmidi_new(codec->card, "ES1370/1", device, &rmidi)) < 0)
+	if ((err = snd_rawmidi_new(codec->card, "CS461X", device, &rmidi)) < 0)
 		return err;
-#ifdef CHIP1370
-	strcpy(rmidi->name, "ES1370");
-#else
-	strcpy(rmidi->name, "ES1371");
-#endif
+	strcpy(rmidi->name, "CS461X");
 	memcpy(&rmidi->chn[SND_RAWMIDI_CHANNEL_OUTPUT].hw, &snd_cs461x_midi_output, sizeof(snd_cs461x_midi_output));
 	memcpy(&rmidi->chn[SND_RAWMIDI_CHANNEL_INPUT].hw, &snd_cs461x_midi_input, sizeof(snd_cs461x_midi_input));
 	rmidi->info_flags |= SND_RAWMIDI_INFO_OUTPUT | SND_RAWMIDI_INFO_INPUT | SND_RAWMIDI_INFO_DUPLEX;
