@@ -281,9 +281,9 @@ static unsigned int snd_es1371_adc_rate(ensoniq_t * ensoniq,
 		rate = 48000;
 	if (rate < 4000)
 		rate = 4000;
-	n = rate / 3000;
+	n = (rate + 1500) / 3000;
 	if ((1 << n) & ((1 << 15) | (1 << 13) | (1 << 11) | (1 << 9)))
-			n--;
+		n--;
 	truncm = (21 * n - 1) | 1;
 	freq = ((48000UL << 15) / rate) * n;
 	result = (48000UL << 15) / (freq / n);
@@ -750,7 +750,7 @@ static snd_pcm_hardware_t snd_ensoniq_playback1 =
 	SND_PCM_CHNINFO_MMAP_VALID |
 	SND_PCM_CHNINFO_PAUSE,	/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* hardware formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000,
 	4000,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
@@ -775,7 +775,7 @@ static snd_pcm_hardware_t snd_ensoniq_playback2 =
 	SND_PCM_CHNINFO_PAUSE,	/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* hardware formats */
 #ifndef CHIP1370
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000,
 #else
 	SND_PCM_RATE_KNOT | 	/* 5512Hz rate */
 	SND_PCM_RATE_11025 | SND_PCM_RATE_22050 | SND_PCM_RATE_44100,
@@ -802,7 +802,7 @@ static snd_pcm_hardware_t snd_ensoniq_capture =
 	SND_PCM_CHNINFO_BLOCK_TRANSFER |
 	SND_PCM_CHNINFO_MMAP_VALID,	/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* hardware formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000,
 	4000,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
@@ -1381,8 +1381,12 @@ int snd_ensoniq_create(snd_card_t * card,
 	if (cmdb < 32)
 		cmdb = 32;
 	pci_write_config_byte(pci, PCI_LATENCY_TIMER, cmdb);
-	pci_read_config_byte(ensoniq->pci, PCI_REVISION_ID, &cmdb);
+	pci_read_config_byte(pci, PCI_REVISION_ID, &cmdb);
 	ensoniq->rev = cmdb;
+	pci_read_config_word(pci, PCI_SUBSYSTEM_VENDOR_ID, &cmdw);
+	ensoniq->subsystem_vendor_id = cmdw;
+	pci_read_config_word(pci, PCI_SUBSYSTEM_ID, &cmdw);
+	ensoniq->subsystem_device_id = cmdw;
 	snd_ensoniq_proc_init(ensoniq);
 #ifdef CHIP1370
 #if 0
@@ -1401,6 +1405,9 @@ int snd_ensoniq_create(snd_card_t * card,
 	ensoniq->ctrl = 0;
 	ensoniq->sctrl = 0;
 	ensoniq->cssr = 0;
+	if (ensoniq->subsystem_vendor_id == 0x107b &&
+	    ensoniq->subsystem_device_id == 0x2150)	/* Gateway Solo 2150 */
+		ensoniq->ctrl |= ES_1371_GPIO_OUT(1);	/* turn amplifier on */
 	/* initialize the chips */
 	outl(ensoniq->ctrl, ES_REG(ensoniq, CONTROL));
 	outl(ensoniq->sctrl, ES_REG(ensoniq, SERIAL));
