@@ -483,9 +483,8 @@ static snd_pcm_hardware_t snd_solo_capture =
 	SND_PCM_CHNINFO_BLOCK | SND_PCM_CHNINFO_INTERLEAVE |
 	SND_PCM_CHNINFO_BLOCK_TRANSFER |
 	SND_PCM_CHNINFO_MMAP_VALID,	/* flags */
-	// SND_PCM_FMT_U8 | SND_PCM_FMT_S8,	/* formats */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_BE | SND_PCM_FMT_S8 | SND_PCM_FMT_U16_BE,	/* formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000,
 	6000,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
@@ -511,7 +510,7 @@ static snd_pcm_hardware_t snd_solo_playback2 =
 	SND_PCM_CHNINFO_BLOCK_TRANSFER |
 	SND_PCM_CHNINFO_MMAP_VALID,	/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE | SND_PCM_FMT_S8 | SND_PCM_FMT_U16_LE,	/* formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000,
 	6000,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
