@@ -421,7 +421,7 @@ static snd_pcm_hardware_t snd_ice1712_playback =
 	SND_PCM_CHNINFO_MMAP_VALID |
 	SND_PCM_CHNINFO_PAUSE,	/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* hardware formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000,
 	4000,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
@@ -444,7 +444,7 @@ static snd_pcm_hardware_t snd_ice1712_capture =
 	SND_PCM_CHNINFO_BLOCK_TRANSFER |
 	SND_PCM_CHNINFO_MMAP_VALID,	/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* hardware formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000,
 	4000,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
