@@ -118,7 +118,7 @@ static void snd_intel8x0_setup_fragments(intel8x0_t *codec, ichdev_t *ichdev)
 	if (ichdev->size == ichdev->fragsize) {
 		ichdev->ack_reload = ichdev->ack = 2;
 		ichdev->fragsize1 = ichdev->fragsize >> 1;
-		for (idx = 0; idx < 64; idx += 4) {
+		for (idx = 0; idx < (ICH_REG_LVI_MASK + 1) * 2; idx += 4) {
 			bdbar[idx + 0] = ichdev->physbuf;
 			bdbar[idx + 1] = 0x80000000 | /* interrupt on completion */
 					 ichdev->fragsize1 >> 1;
@@ -130,16 +130,21 @@ static void snd_intel8x0_setup_fragments(intel8x0_t *codec, ichdev_t *ichdev)
 	} else {
 		ichdev->ack_reload = ichdev->ack = 1;
 		ichdev->fragsize1 = ichdev->fragsize;
-		for (idx = 0; idx < 64; idx += 2) {
+		for (idx = 0; idx < (ICH_REG_LVI_MASK + 1) * 2; idx += 2) {
 			bdbar[idx + 0] = ichdev->physbuf + (((idx >> 1) * ichdev->fragsize) % ichdev->size);
 			bdbar[idx + 1] = 0x80000000 | /* interrupt on completion */
 					 ichdev->fragsize >> 1;
+			// printk("bdbar[%i] = 0x%x [0x%x]\n", idx + 0, bdbar[idx + 0], bdbar[idx + 1]);
 		}
 		ichdev->frags = ichdev->size / ichdev->fragsize;
 	}
-	outb(ichdev->lvi = 31, port + ICH_REG_PI_LVI);
-	ichdev->lvi_frag = (32 - 1) % ichdev->frags;
+	outb(ichdev->lvi = ICH_REG_LVI_MASK, port + ICH_REG_PI_LVI);
+	ichdev->lvi_frag = ICH_REG_LVI_MASK % ichdev->frags;
 	ichdev->position = 0;
+#if 0
+	printk("lvi_frag = %i, frags = %i, frag_size = 0x%x, frag_size1 = 0x%x\n",
+			ichdev->lvi_frag, ichdev->frags, ichdev->fragsize, ichdev->fragsize1);
+#endif
 	/* clear interrupts */
 	outb(ICH_FIFOE | ICH_BCIS | ICH_LVBCI, port + ICH_REG_PI_SR);
 }
@@ -163,7 +168,8 @@ static inline void snd_intel8x0_update(intel8x0_t *codec, ichdev_t *ichdev)
 	ichdev->lvi_frag++;
 	ichdev->lvi_frag %= ichdev->frags;
 	ichdev->bdbar[ichdev->lvi * 2] = ichdev->physbuf + ichdev->lvi_frag * ichdev->fragsize1;
-	if ((ack = --ichdev->ack == 0) != 0)
+	// printk("new: bdbar[%i] = 0x%x [0x%x], prefetch = %i, all = 0x%x, 0x%x\n", ichdev->lvi * 2, ichdev->bdbar[ichdev->lvi * 2], ichdev->bdbar[ichdev->lvi * 2 + 1], inb(ICH_REG_PI_PIV + port), inl(port + 4), inb(port + ICH_REG_PI_CR));
+	if ((ack = (--ichdev->ack == 0)) != 0)
 		ichdev->ack = ichdev->ack_reload;
 	spin_unlock_irqrestore(&codec->reg_lock, flags);
 	if (ack && ichdev->subchn)
@@ -246,9 +252,17 @@ static int snd_intel8x0_playback_prepare(void *private_data,
 	codec->playback.physbuf = virt_to_bus(runtime->dma_area->buf);
 	codec->playback.size = snd_pcm_lib_transfer_size(subchn);
 	codec->playback.fragsize = snd_pcm_lib_transfer_fragment(subchn);
-	spin_lock_irqsave(&codec->ac97->reg_lock, flags);
-	snd_intel8x0_codec_write(codec, AC97_PCM_FRONT_DAC_RATE, runtime->format.rate);
-	spin_unlock_irqrestore(&codec->ac97->reg_lock, flags);
+	/* disable DAC power */
+	snd_ac97_write_bitmask_lock(codec->ac97, AC97_POWERDOWN, ~0x0200, 0x0200);
+	snd_ac97_write_lock(codec->ac97, AC97_PCM_FRONT_DAC_RATE, runtime->format.rate);
+	/* enable DAC power */
+	snd_ac97_write_bitmask_lock(codec->ac97, AC97_POWERDOWN, ~0x0200, 0x0000);
+#if 0
+	printk("prepare: AC97_PCM_FRONT_DAC_RATE=0x%x (%i), AC97_EXTENDED_STATUS=0x%x\n",
+		snd_ac97_read_lock(codec->ac97, AC97_PCM_FRONT_DAC_RATE),
+		snd_ac97_read_lock(codec->ac97, AC97_PCM_FRONT_DAC_RATE),
+		snd_ac97_read_lock(codec->ac97, AC97_EXTENDED_STATUS));
+#endif
 	spin_lock_irqsave(&codec->reg_lock, flags);
 	snd_intel8x0_setup_fragments(codec, &codec->playback);
 	spin_unlock_irqrestore(&codec->reg_lock, flags);
@@ -265,9 +279,11 @@ static int snd_intel8x0_capture_prepare(void *private_data,
 	codec->capture.physbuf = virt_to_bus(runtime->dma_area->buf);
 	codec->capture.size = snd_pcm_lib_transfer_size(subchn);
 	codec->capture.fragsize = snd_pcm_lib_transfer_fragment(subchn);
-	spin_lock_irqsave(&codec->ac97->reg_lock, flags);
-	snd_intel8x0_codec_write(codec, AC97_PCM_LR_ADC_RATE, runtime->format.rate);
-	spin_unlock_irqrestore(&codec->ac97->reg_lock, flags);
+	/* disable ADC power */
+	snd_ac97_write_bitmask_lock(codec->ac97, AC97_POWERDOWN, ~0x0100, 0x0100);
+	snd_ac97_write_lock(codec->ac97, AC97_PCM_LR_ADC_RATE, runtime->format.rate);
+	/* enable ADC power */
+	snd_ac97_write_bitmask_lock(codec->ac97, AC97_POWERDOWN, ~0x0100, 0x0000);
 	spin_lock_irqsave(&codec->reg_lock, flags);
 	snd_intel8x0_setup_fragments(codec, &codec->capture);
 	spin_unlock_irqrestore(&codec->reg_lock, flags);
@@ -310,8 +326,8 @@ static snd_pcm_hardware_t snd_intel8x0_playback =
 	SND_PCM_CHNINFO_MMAP_VALID |
 	SND_PCM_CHNINFO_PAUSE,	/* flags */
 	SND_PCM_FMT_S16_LE,	/* hardware formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
-	4000,			/* min. rate */
+	0,			/* overwritten */
+	8000,			/* min. rate */
 	48000,			/* max. rate */
 	2,			/* min. voices */
 	2,			/* max. voices */
@@ -333,8 +349,8 @@ static snd_pcm_hardware_t snd_intel8x0_capture =
 	SND_PCM_CHNINFO_BLOCK_TRANSFER |
 	SND_PCM_CHNINFO_MMAP_VALID,	/* flags */
 	SND_PCM_FMT_S16_LE,	/* hardware formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
-	4000,			/* min. rate */
+	0,			/* overwritten */
+	8000,			/* min. rate */
 	48000,			/* max. rate */
 	2,			/* min. voices */
 	2,			/* max. voices */
@@ -367,6 +383,8 @@ static int snd_intel8x0_playback_open(void *private_data,
 	codec->playback.subchn = subchn;
 	memcpy(hw, &snd_intel8x0_playback, sizeof(*hw));
 	hw->rates = codec->playback.rates;
+	if (!(hw->rates & SND_PCM_RATE_8000))
+		hw->min_rate = 48000;
 	subchn->runtime->hw = hw;
 	subchn->runtime->hw_free = _snd_kfree;
 	snd_pcm_set_mixer(subchn, codec->mixer->device, codec->ac97->me_playback);
@@ -391,6 +409,8 @@ static int snd_intel8x0_capture_open(void *private_data,
 	codec->capture.subchn = subchn;
 	memcpy(hw, &snd_intel8x0_capture, sizeof(*hw));
 	hw->rates = codec->capture.rates;
+	if (!(hw->rates & SND_PCM_RATE_8000))
+		hw->min_rate = 48000;
 	subchn->runtime->hw = hw;
 	subchn->runtime->hw_free = _snd_kfree;
 	snd_pcm_set_mixer(subchn, codec->mixer->device, codec->ac97->me_capture);
@@ -404,6 +424,8 @@ static int snd_intel8x0_playback_close(void *private_data,
 
 	codec->playback.subchn = NULL;
 	snd_pcm_dma_free(subchn);
+	/* disable DAC power */
+	snd_ac97_write_bitmask_lock(codec->ac97, AC97_POWERDOWN, ~0x0200, 0x0200);
 	return 0;
 }
 
@@ -414,6 +436,8 @@ static int snd_intel8x0_capture_close(void *private_data,
 
 	codec->capture.subchn = NULL;
 	snd_pcm_dma_free(subchn);
+	/* disable ADC power */
+	snd_ac97_write_bitmask_lock(codec->ac97, AC97_POWERDOWN, ~0x0100, 0x0100);
 	return 0;
 }
 
@@ -487,9 +511,11 @@ static int snd_intel8x0_capture_mic_prepare(void *private_data,
 	codec->capture_mic.physbuf = virt_to_bus(runtime->dma_area->buf);
 	codec->capture_mic.size = snd_pcm_lib_transfer_size(subchn);
 	codec->capture_mic.fragsize = snd_pcm_lib_transfer_fragment(subchn);
-	spin_lock_irqsave(&codec->ac97->reg_lock, flags);
-	snd_intel8x0_codec_write(codec, AC97_PCM_MIC_ADC_RATE, runtime->format.rate);
-	spin_unlock_irqrestore(&codec->ac97->reg_lock, flags);
+	/* disable ADC power */
+	snd_ac97_write_bitmask_lock(codec->ac97, AC97_EXTENDED_STATUS, ~0x4000, 0x4000);
+	snd_ac97_write_lock(codec->ac97, AC97_PCM_MIC_ADC_RATE, runtime->format.rate);
+	/* enable ADC power */
+	snd_ac97_write_bitmask_lock(codec->ac97, AC97_EXTENDED_STATUS, ~0x4000, 0x0000);
 	spin_lock_irqsave(&codec->reg_lock, flags);
 	snd_intel8x0_setup_fragments(codec, &codec->capture_mic);
 	spin_unlock_irqrestore(&codec->reg_lock, flags);
@@ -517,10 +543,10 @@ static snd_pcm_hardware_t snd_intel8x0_capture_mic =
 	SND_PCM_CHNINFO_BLOCK_TRANSFER |
 	SND_PCM_CHNINFO_MMAP_VALID |
 	SND_PCM_CHNINFO_PAUSE,	/* flags */
-	SND_PCM_FMT_S32_LE,	/* hardware formats */
-	SND_PCM_RATE_KNOT | SND_PCM_RATE_8000_96000,
-	4000,			/* min. rate */
-	96000,			/* max. rate */
+	SND_PCM_FMT_S16_LE,	/* hardware formats */
+	0,			/* overwritten */
+	8000,			/* min. rate */
+	48000,			/* max. rate */
 	1,			/* min. voices */
 	1,			/* max. voices */
 	32,			/* min. fragment size */
@@ -552,6 +578,8 @@ static int snd_intel8x0_capture_mic_open(void *private_data,
 	codec->capture_mic.subchn = subchn;
 	memcpy(hw, &snd_intel8x0_capture_mic, sizeof(*hw));
 	hw->rates = codec->capture_mic.rates;
+	if (!(hw->rates & SND_PCM_RATE_8000))
+		hw->min_rate = 48000;
 	subchn->runtime->hw = hw;
 	subchn->runtime->hw_free = _snd_kfree;
 	return 0;
@@ -564,6 +592,8 @@ static int snd_intel8x0_capture_mic_close(void *private_data,
 
 	codec->capture_mic.subchn = NULL;
 	snd_pcm_dma_free(subchn);
+	/* disable ADC power */
+	snd_ac97_write_bitmask_lock(codec->ac97, AC97_EXTENDED_STATUS, ~0x4000, 0x4000);
 	return 0;
 }
 
@@ -600,11 +630,16 @@ int snd_intel8x0_pcm_mic(intel8x0_t * codec, int device, snd_pcm_t ** rpcm)
  *  Mixer part
  */
 
-static void snd_intel8x0_codec_init(void *private_data)
+static void snd_intel8x0_codec_init(void *private_data, ac97_t *ac97)
 {
 	intel8x0_t *codec = snd_magic_cast(intel8x0_t, private_data, );
 
-	snd_intel8x0_codec_write(codec, AC97_EXTENDED_STATUS, 0x0009);	
+	if (codec->ac97_ext_id & 0x0001)	/* VRA */
+		snd_ac97_write_lock(ac97, AC97_EXTENDED_STATUS, 0x0009);	
+	/* disable DAC & ADC power */
+	snd_ac97_write_bitmask_lock(ac97, AC97_POWERDOWN, ~0x0300, 0x0300);
+	/* disable center DAC/surround DAC/LFE DAC/MIC ADC */
+	snd_ac97_write_bitmask_lock(ac97, AC97_EXTENDED_STATUS, (short)~0xe800, 0xe800);
 }
 
 static void snd_intel8x0_mixer_free_ac97(void *private_data)
@@ -653,7 +688,7 @@ static void snd_intel8x0_determine_rates(intel8x0_t *codec, int reg, unsigned in
 
 	/* test a non-standard frequency */
 	if (snd_intel8x0_test_rate(codec, reg, 11000))
-		result |= SND_PCM_RATE_PLL;
+		result |= SND_PCM_RATE_CONTINUOUS;
 	/* let's try to obtain standard frequencies */
 	if (snd_intel8x0_test_rate(codec, reg, 8000))
 		result |= SND_PCM_RATE_8000;
@@ -682,15 +717,19 @@ static int snd_intel8x0_chip_init(intel8x0_t *codec)
 		snd_printk("snd_intel8x0_chip_init: invalid AC'97 codec\n");
 		return -EIO;
 	}
-	if (!(ext_id & 0x0001))	{ /* VRA */
-		snd_printk("snd_intel8x0_chip_init: AC'97 doesn't support VRA\n");
-		return -EIO;
+	if (ext_id & 0x0001) {
+		snd_intel8x0_codec_write(codec, AC97_EXTENDED_STATUS, 0x0009);
+		snd_intel8x0_determine_rates(codec, AC97_PCM_FRONT_DAC_RATE, &codec->playback.rates);
+		snd_intel8x0_determine_rates(codec, AC97_PCM_LR_ADC_RATE, &codec->capture.rates);
+	} else {
+		codec->playback.rates = SND_PCM_RATE_48000;
+		codec->capture.rates = SND_PCM_RATE_48000;
 	}
-	snd_intel8x0_codec_write(codec, AC97_EXTENDED_STATUS, 0x0009);
-	snd_intel8x0_determine_rates(codec, AC97_PCM_FRONT_DAC_RATE, &codec->playback.rates);
-	snd_intel8x0_determine_rates(codec, AC97_PCM_LR_ADC_RATE, &codec->capture.rates);
-	if (ext_id & 0x0008)
+	if (ext_id & 0x0008) {
 		snd_intel8x0_determine_rates(codec, AC97_PCM_MIC_ADC_RATE, &codec->capture_mic.rates);
+	} else {
+		codec->capture_mic.rates = SND_PCM_RATE_48000;
+	}
 	/* disable interrupts */
 	outb(0x00, ICHREG(codec, PI_CR));
 	outb(0x00, ICHREG(codec, PO_CR));
