@@ -104,7 +104,7 @@ static snd_pcm_hardware_t snd_rme9652_playback_subchannel_info =
 
 	SND_PCM_FMT_S32_LE,     /* formats */
 
-	SND_PCM_RATE_PLL | 
+	SND_PCM_RATE_KNOT | 	/* 64kHz */
 	SND_PCM_RATE_44100 | 
 	SND_PCM_RATE_48000 | 
 	SND_PCM_RATE_88200 | 
@@ -141,13 +141,13 @@ static snd_pcm_hardware_t snd_rme9652_capture_subchannel_info =
 	SND_PCM_CHNINFO_PAUSE |
 	SND_PCM_CHNINFO_NONINTERLEAVE, /* modes */
 
-	SND_PCM_FMT_S32_LE,    /* formats */
+	SND_PCM_FMT_S32_LE,	/* formats */
 
-	SND_PCM_RATE_PLL | 
+	SND_PCM_RATE_KNOT |	/* 64kHz */
 	SND_PCM_RATE_44100 | 
 	SND_PCM_RATE_48000 | 
 	SND_PCM_RATE_88200 | 
-	SND_PCM_RATE_96000,    /* rates */
+	SND_PCM_RATE_96000,	/* rates */
 
 	44100,			/* min. rate (can go lower, but not
 				   via s/w control)
