@@ -479,7 +479,7 @@ static snd_pcm_hardware_t snd_sonicvibes_playback =
 	SND_PCM_CHNINFO_BLOCK_TRANSFER |
 	SND_PCM_CHNINFO_MMAP_VALID,
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000,
 	4000,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
@@ -502,7 +502,7 @@ static snd_pcm_hardware_t snd_sonicvibes_capture =
 	SND_PCM_CHNINFO_BLOCK_TRANSFER |
 	SND_PCM_CHNINFO_MMAP_VALID,
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000,
 	4000,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
