@@ -1470,7 +1470,7 @@ static snd_pcm_hardware_t snd_trident_playback =
 	SND_PCM_CHNINFO_MMAP_VALID,		/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE |
 	SND_PCM_FMT_S8 | SND_PCM_FMT_U16_LE,	/* formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000, /* supported RATES */
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000, /* supported RATES */
 	4000,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
@@ -1498,7 +1498,7 @@ static snd_pcm_hardware_t snd_trident_capture =
 	SND_PCM_CHNINFO_MMAP_VALID,		/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE |
 	SND_PCM_FMT_S8 | SND_PCM_FMT_U16_LE,	/* formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000, /* supported RATES */
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000, /* supported RATES */
 	4000,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
