@@ -385,7 +385,7 @@ static snd_pcm_hardware_t snd_sb16_playback =
 	SND_PCM_CHNINFO_BLOCK | SND_PCM_CHNINFO_INTERLEAVE |
 	SND_PCM_CHNINFO_MMAP_VALID,	/* flags */
 	0,			/* formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_44100,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_44100,
 	4000,			/* min. rate */
 	44100,			/* max. rate */
 	1,			/* min. voices */
@@ -407,7 +407,7 @@ static snd_pcm_hardware_t snd_sb16_capture =
 	SND_PCM_CHNINFO_BLOCK | SND_PCM_CHNINFO_INTERLEAVE |
 	SND_PCM_CHNINFO_MMAP_VALID,	/* flags */
 	0,			/* formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_44100,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_44100,
 	4000,			/* min. rate */
 	44100,			/* max. rate */
 	1,			/* min. voices */
